#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <bootrecord.h>



/* 
   Routine to print stars at the reserved areas of the 
   boot record
*/
void
Stars(char *str,int n){
	int i;
	for(i=0;i<n;i++){
		str[i]='*';
	}
}

/*
   Integer byte swapping routine (4-bytes)
*/
int
intbyteswap(int passed_val){
  // Byte swap, little endian->big endian
  int return_val;
  char *s1,*s2;

#ifdef LITTLE_ENDIAN
  s1=(char*)&passed_val;
  s2=(char*)&return_val;
  s2[0] = s1[3];
  s2[1] = s1[2];
  s2[2] = s1[1];
  s2[3] = s1[0];
#endif LITTLE_ENDIAN

#ifdef BIG_ENDIAN
/* We may want to do things here*/
   return passed_val;
#endif BIG_ENDIAN

  return return_val;
}

/*
   Short byte swapping routine (2-bytes)
*/
short
shortbyteswap(short passed_val){
  // Byte swap, little endian->big endian
  short return_val;
  char *s1,*s2;
#ifdef LITTLE_ENDIAN
  s1=(char*)&passed_val;
  s2=(char*)&return_val;
  s2[0] = s1[1];
  s2[1] = s1[0];
#endif LITTLE_ENDIAN

#ifdef BIG_ENDIAN
/* We may want to do things here*/
   return passed_val;
#endif BIG_ENDIAN

  return return_val;
}


/* Main writes out the IPL_CB for a boot disk to stdout.
   It byte swaps for a little endian machine
   as RS6K's are big endian.
   Under linux LITTLE_ENDIAN is already defined 
   in the gcc compiling environment, so you DONT
   define it when compiling. Other environments
   may need it, some may not; experiment.
   */

int
main(){

IPL_REC ipl_rec_area;


   ipl_rec_area.IPL_record_id		=intbyteswap(IPLRECID);		
                                              /* This physical volume contains a   */
                                              /* valid IPL record if and only if   */
                                              /* this field contains IPLRECID      */

	Stars(ipl_rec_area.reserved1,20);
	
	ipl_rec_area.formatted_cap 	=intbyteswap(2880);    		
                                              /* Formatted capacity. The number of */
                                              /* sectors available after formatting*/
                                              /* The presence or absence of bad    */
                                       	 		  /* blocks does not alter this value. */
																              /* One sector seems to be 				 */
																              /* 2 * K, providied 1000K = 1 Meg	 */
																
	ipl_rec_area.last_head			=1;        		/* THIS IS DISKETTE INFORMATION      */
                                        			/* The number of heads minus 1. Heads*/
                                        			/* are number from 0 to last_head.   */
																              /* Normal Diskette in 250 and 320H   */
																              /* Have two heads							 */
																			
	ipl_rec_area.last_sector		=18;      		/* THIS IS DISKETTE INFORMATION      */
                                        			/* The number of sectors per track.  */
                                        			/* Sectors are numbered from 1 to    */
                                        			/* last_sector.                      */

	Stars(ipl_rec_area.reserved2,6);

	ipl_rec_area.boot_code_length	= intbyteswap(4); 				
	
																              /* Boot code length in sectors. A 0  */
                                        			/* value implies no boot code present*/

	ipl_rec_area.boot_code_offset = intbyteswap(2); 			

																              /* Boot code offset. Must be 0 if no  */
                                        			/* boot code present, else contains   */
                                        			/* byte offset from start of boot     */
                                        			/* code to first instruction.         */
																              /* Measured in bytes.				          */
																
	ipl_rec_area.boot_lv_start		= intbyteswap(0);    			
	
																              /* Contains the PSN of the start of  */
                                        			/* the BLV.                          */
																              /* For a floppy it's in the first 	 */
																              /* sector, i.e. sector zero.		    */
																
	ipl_rec_area.boot_prg_start	= intbyteswap(1);   			
	
																              /* Boot code start. Must be 0 if no  */
 																              /* boot code present, else contains  */
                                        			/* the PSN of the start of boot code.*/

	ipl_rec_area.boot_lv_length	= intbyteswap(4);   			
	
																              /* BLV length in sectors.            */

	ipl_rec_area.boot_load_add		= intbyteswap(512);    		
	
																              /* 512 byte boundary load address for*/
                                        			/* boot code.                        */

	ipl_rec_area.boot_frag			= 0;        	/* Boot code fragmentation flag. Must*/
                                        			/* be 0 if no fragmentation allowed, */
                                        			/* else must be 0x01.                */

	Stars(ipl_rec_area.reserved3,3);

	ipl_rec_area.basecn_length		=shortbyteswap(0);    			
                                              /* Number of sectors for base        */
                                        			/* customization. Normal mode.       */

	ipl_rec_area.basecs_length		=shortbyteswap(0);    			
                                              /* Number of sectors for base        */
                                        			/* customization. Service mode.      */

	ipl_rec_area.basecn_start		=intbyteswap(0);     			
                                              /* Starting PSN value for base       */
                                        			/* customization. Normal mode.       */

	ipl_rec_area.basecs_start		=intbyteswap(0);     			
                                              /* Starting PSN value for base       */
                                        			/* customization. Service mode.      */

	Stars(ipl_rec_area.reserved4,24);

	ipl_rec_area.ser_code_length	=intbyteswap(4);  				
                                              /* Service code length in sectors.   */
                                      			  /* A 0 value implies no service code */
                                      				/* present.                          */

	ipl_rec_area.ser_code_offset  =intbyteswap(0);  			 
                                              /* Service code offset. Must be 0 if        */
                                        			/* no service code is present (AJR hmm...   */
                                              /* I don't think so! Also it's the number   */
                                              /* of bytes clipped to long word boundaries */
                                              /* by a floor() like function), else        */
                                        			/* contains byte offset from start of       */
                                        			/* service code to first instruction.       */

	ipl_rec_area.ser_lv_start		=intbyteswap(0);     			
                                              /* Contains the PSN of the start of  */
                                        			/* the SLV.                          */

	ipl_rec_area.ser_prg_start		=intbyteswap(1);   			
                                              /* Service code start. Must be 0 if  */
                                        			/* service code is not present, else */
                                        			/* contains the PSN of the start of  */
                                        			/* service code.                     */

	ipl_rec_area.ser_lv_length		=intbyteswap(4);    			
                                              /* SLV length in sectors.            */

	ipl_rec_area.ser_load_add     =intbyteswap(512); 			
                                              /* 512 byte boundary load address for*/
                                        			/* service code.                     */

	ipl_rec_area.ser_frag			=0;         	/* Service code fragmentation flag.  */
                                        			/* Must be 0 if no fragmentation     */
                                        			/* allowed, else must be 0x01.       */


  Stars(ipl_rec_area.reserved5,3);
  
  ipl_rec_area.pv_id.word1=0x11111111;        /* Unique ID of this physical volume*/
  ipl_rec_area.pv_id.word2=0x11111111;
  ipl_rec_area.pv_id.word3=0x11111111;
  ipl_rec_area.pv_id.word4=0x11111111;
  
  Stars(ipl_rec_area.dummy,(512 - 128 - sizeof(unique_id_t)));
                                              /* Padding */  
  
  /* Write out the data to stdout */
  write(1,&ipl_rec_area,sizeof(IPL_REC));
  
}
